//********************** x12demo.c ****************************
//* This file contains demo (test) program for Sensoray model
//* 512 MPEG codec.
//* Written by Alexander Kostromitin
//* Copyright (C) Sensoray Company 2001
//*************************************************************

#define __MSC__		// use MicroSoft C compiler

#include <time.h>
#include <string.h>
#include <stdio.h>
#include <windows.h>
#include <commdlg.h>    // includes common dialog functionality
#include <commctrl.h>
#include <dlgs.h>       // includes common dialog template defines
#include <cderr.h>      // includes the common dialog error codes
#include <io.h>

#ifndef __MSC__
#include <dir.h>
#else
#define	MAXPATH	256
#endif

#include "x12demo.h"
#include "sx12.h"
#include "sx12f.h"

int APIENTRY MainWndProc(HWND hDlg, int uMsg, WPARAM wParam, LPARAM lParam);
BOOL InitApplication(void);
BOOL InitInstance(void);
BOOL APIENTRY startdlg(HWND hDlg,UINT message,UINT wParam,LONG lParam);

void readwrite(void);
int getfilename(char *name,char *title);
int error(char *hdr,int key,char *fmt, ... );
 

HINSTANCE MainInst;		// the current instance
HWND      MainWin;              // hWnd of main window

char *AppName="x12demo";
char *AppHdr="x12demo";

int num_cards,card=0;
int sys,br,vbr,sz,innum,intype,mode,pause;
char fname[MAXPATH] = "tmp.mpg";
int starttime,rtime,rlen;
int endkey;
FILE *file;
char buffer[1024*32];		// maximum buffer size is 32k

enum modes { STOP, PAUSE, PLAY, WRITE };
char *mnames[] = { "STOP", "PAUSE", "PLAY", "WRITE" };

HANDLE thandle;
DWORD threadID;


#ifndef __MSC__
#pragma argsused
#endif

int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,
 LPSTR lpCmdLine,int nCmdShow)
{
MSG msg;
int err;

err=X12_DLLOpen();   		// open DLL; see sx12app.c
if(err)
  {
  error(AppName,MB_ICONERROR|MB_SYSTEMMODAL,"DLL open error #%d",err);
  return(0);
  }
num_cards=X12_init();        	// init 512 cards
if(num_cards<0)
  {
  error(AppName,MB_ICONERROR|MB_SYSTEMMODAL,"Driver open error");
  return(0);                  	// problem with driver
  }
if(num_cards==0)
  {
  error(AppName,MB_ICONERROR|MB_SYSTEMMODAL,"No x12 devices found");
  return(0);                    // no 512 device or incorrect I/O port selected
  }
MainInst=hInstance;
if(!InitApplication()) return(FALSE);
if(!InitInstance()) return(FALSE);
while(GetMessage(&msg,NULL,0,0))
  {
  TranslateMessage(&msg);
  DispatchMessage(&msg);
  }
X12_DLLClose();			// close DLL
return(msg.wParam);
}


BOOL InitApplication(void)
{
WNDCLASS wc;

sys=NTSC; br=1000000; vbr=0; sz=SIZE11; mode=STOP; intype=CVIDEO; innum=0;

wc.style=0;
wc.lpfnWndProc=(WNDPROC)MainWndProc;
wc.cbClsExtra=0;
wc.cbWndExtra=0;
wc.hInstance=MainInst;
wc.hIcon=LoadIcon(MainInst,"MainIcon");
wc.hCursor=0;
wc.hbrBackground=0;
wc.lpszMenuName=0;
wc.lpszClassName=AppName;
return(RegisterClass(&wc));
}


BOOL InitInstance(void)
{
//InitCommonControls();
MainWin=CreateWindow(
    AppName,
    AppHdr,
    WS_OVERLAPPEDWINDOW,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0,
    0,
    NULL,
    NULL,
    MainInst,
    NULL);
if(!MainWin) return(FALSE);
return(TRUE);
}


#ifndef __MSC__
#pragma argsused
#endif

int APIENTRY MainWndProc(HWND hDlg, int uMsg, WPARAM wParam, LPARAM lParam)
{
switch(uMsg)
  {
  case WM_CREATE:
    MainWin=hDlg;
    DialogBox(MainInst,"Edit",MainWin,startdlg);
    break;
  case WM_DESTROY:
    PostQuitMessage(0);
    break;
  default: return(DefWindowProc(hDlg,uMsg,wParam,lParam));
  }
return(0);
}


#ifndef __MSC__
#pragma argsused
#endif

BOOL APIENTRY startdlg(HWND hDlg,UINT message,UINT wParam,LONG lParam)
{
int cmd;
int newmode;
BOOL ok;
int tmp;

switch(message)
  {
  case WM_INITDIALOG:
    SendMessage(hDlg,WM_SETICON,0,(LPARAM)LoadIcon(MainInst,"MainIcon"));
    MainWin=hDlg;
    SetDlgItemText(hDlg,IDC_FILE,fname);
    SetDlgItemText(hDlg,IDC_MODE,mnames[mode]);
    SetDlgItemInt(hDlg,IDC_RATE,br,0);
    SetDlgItemInt(hDlg,IDC_TDISP,rtime,0);
    SetDlgItemInt(hDlg,IDC_LDISP,rlen,0);
    SendDlgItemMessage(hDlg,IDC_NTSC,BM_SETCHECK,sys==NTSC,0);
    SendDlgItemMessage(hDlg,IDC_PAL,BM_SETCHECK,sys==PAL,0);

    SendDlgItemMessage(hDlg,IDC_COMPOSITE,BM_SETCHECK,intype==CVIDEO,0);
    SendDlgItemMessage(hDlg,IDC_SVIDEO,BM_SETCHECK,intype==SVIDEO,0);

    SendDlgItemMessage(hDlg,IDC_IN1,BM_SETCHECK,innum==0,0);
    SendDlgItemMessage(hDlg,IDC_IN2,BM_SETCHECK,innum==1,0);
    SendDlgItemMessage(hDlg,IDC_IN3,BM_SETCHECK,innum==2,0);
    SendDlgItemMessage(hDlg,IDC_IN4,BM_SETCHECK,innum==3,0);

    SendDlgItemMessage(hDlg,IDC_SIZE11,BM_SETCHECK,sz==SIZE11,0);
    SendDlgItemMessage(hDlg,IDC_SIZE23,BM_SETCHECK,sz==SIZE23,0);
    SendDlgItemMessage(hDlg,IDC_SIZE12,BM_SETCHECK,sz==SIZE12,0);
    SendDlgItemMessage(hDlg,IDC_SIZE_1,BM_SETCHECK,sz==SIZE_1,0);
    SendDlgItemMessage(hDlg,IDC_VBR,BM_SETCHECK,vbr,0);
    if((thandle=CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)readwrite,
	NULL,0,&threadID))==0) return(FALSE);
    break;
  case WM_DESTROY:
    break;
  case WM_COMMAND:
    switch(cmd=LOWORD(wParam))
      {
      case IDC_BROWSE:
	GetDlgItemText(hDlg,IDC_FILE,fname,MAXPATH-1);
	if(getfilename(fname,"Open file")==0) break;
	SetDlgItemText(hDlg,IDC_FILE,fname);
	break;
      case IDC_PAUSE:
	pause=!pause;
	if(pause) SetDlgItemText(hDlg,IDC_MODE,"PAUSE");
	else SetDlgItemText(hDlg,IDC_MODE,mnames[mode]);
	break;
      case IDC_PLAY:
      case IDC_WRITE:
	newmode=cmd==IDC_PLAY?PLAY:WRITE;
	if(mode==newmode) return(FALSE);
	if(newmode==PLAY && access(fname,4)) return(FALSE);
	GetDlgItemText(hDlg,IDC_FILE,fname,MAXPATH-1);
	tmp=GetDlgItemInt(hDlg,IDC_RATE,&ok,0);
	if(!ok || tmp<=0 || tmp>200000000) return(FALSE);
	br=tmp;
	if(SendDlgItemMessage(hDlg,IDC_NTSC,BM_GETCHECK,0,0)) sys=NTSC;
	else sys=PAL;

	if(SendDlgItemMessage(hDlg,IDC_COMPOSITE,BM_GETCHECK,0,0)) intype=CVIDEO;
	else intype=SVIDEO;

	if(SendDlgItemMessage(hDlg,IDC_IN1,BM_GETCHECK,0,0)) innum=0;
	else if(SendDlgItemMessage(hDlg,IDC_IN2,BM_GETCHECK,0,0)) innum=1;
	else if(SendDlgItemMessage(hDlg,IDC_IN3,BM_GETCHECK,0,0)) innum=2;
	else innum=3;

	if(SendDlgItemMessage(hDlg,IDC_SIZE11,BM_GETCHECK,0,0)) sz=SIZE11;
	else if(SendDlgItemMessage(hDlg,IDC_SIZE23,BM_GETCHECK,0,0)) sz=SIZE23;
	else if(SendDlgItemMessage(hDlg,IDC_SIZE12,BM_GETCHECK,0,0)) sz=SIZE12;
	else sz=SIZE_1;
	vbr=SendDlgItemMessage(hDlg,IDC_VBR,BM_GETCHECK,0,0);
	if(newmode==WRITE)      // open file to write
	  {
	  if((file=fopen(fname,"wb"))==NULL)
	    {
	    error(AppName,MB_ICONERROR|MB_SYSTEMMODAL,
	      "Open error of file %s",fname);
	    return(FALSE);
	    }
	  }
	else  			// open file to read
	  {
	  if((file=fopen(fname,"rb"))==NULL)
	    {
	    error(AppName,MB_ICONERROR|MB_SYSTEMMODAL,
	      "File %s not found",fname);
	    return(FALSE);
	    }
	  }
	X12_set_system(card,sys);	// set video system
	X12_set_bitrate(card,br);       // set bitrate
	X12_set_picsize(card,sz);       // set picture format and size
	X12_set_input(card,innum);        // set input
	X12_set_input_type(card,intype); // set input type
	X12_set_vbr(card,vbr);          // set bitrate
	X12_reset(card);                // send paremeters to card

	starttime=time(0); rlen=0; pause=0;
	SetDlgItemText(hDlg,IDC_MODE,mnames[mode=newmode]);

	break;
      case IDC_STOP:
	newmode=STOP;
	SetDlgItemText(hDlg,IDC_MODE,mnames[newmode]);
	if(mode==newmode) return(FALSE);
	Sleep(1000);				// wait for read/write completed
	if(file) { fclose(file); file=0; }      // close file
	mode=newmode;
	break;

      case IDOK:
      case IDCANCEL:
	mode=STOP;
	endkey=1;
	WaitForSingleObject(thandle,3000);      // wait for thread completed
	X12_cleanup();                          // close device
	PostQuitMessage(0);
	break;
      }
    break;
  default: return(FALSE);
  }
return(TRUE);
}


void readwrite(void)
{
int oldmode=0,len,t;
char sbuf[32];

while(!endkey)
  {
  if(oldmode==mode && mode!=PLAY && mode!=WRITE || pause)
    { Sleep(50); continue; }			// no actions
  if(mode==WRITE)                               // WRITE mode
    {
    len=X12_read(card,buffer,sizeof(buffer));   // read from device
    if(len<0)
      {
      SetDlgItemText(MainWin,IDC_MODE,"No signal");
      mode=STOP;
      continue;
      }
    if(len<sizeof(buffer))
      {
      Sleep(10);
      if(len==0) continue;                      // not ready: repeat
      }
    rlen+=len;
    fwrite(buffer,1,len,file);                  // write to file
    sprintf(sbuf,"%d.%03d.%03d",rlen/1000000,rlen/1000%1000,rlen%1000);
    SetDlgItemText(MainWin,IDC_LDISP,sbuf);
    }
  else if(mode==PLAY)				// PLAY mode
    {
    len=fread(buffer,1,sizeof(buffer),file);	// read from file
    if(len==0)                                  // end of file
     {
     mode=STOP;
     SetDlgItemText(MainWin,IDC_MODE,mnames[mode]);
     }
    else
      {
      int nb; char *bp;
      for(bp=buffer;len>0 && mode==PLAY;)
        {
        nb=X12_write(card,bp,len);              // write to device
        if(nb<len)
          Sleep(10);
        bp+=nb; len-=nb;
        rlen+=nb;
        }
      sprintf(sbuf,"%d.%03d.%03d",rlen/1000000,rlen/1000%1000,rlen%1000);
      SetDlgItemText(MainWin,IDC_LDISP,sbuf);
      }
    }
  t=time(0)-starttime;
  if(t!=rtime) SetDlgItemInt(MainWin,IDC_TDISP,rtime=t,0);

  oldmode=mode;
  }

}


int getfilename(char *name,char *title)		// browse file
{
static OPENFILENAME ofninfo;
int err;
char ff[]="VideoFiles (*.mpg)\0*.mpg\0AllFiles (*.*)\0*.*\0";

ofninfo.lStructSize=sizeof(OPENFILENAME);
ofninfo.hwndOwner=MainWin;
ofninfo.hInstance=0;
ofninfo.lpstrFilter=(LPSTR)ff;
ofninfo.lpstrCustomFilter=NULL;
ofninfo.nMaxCustFilter=0;
ofninfo.nFilterIndex=1;
ofninfo.lpstrFile=(LPSTR)name;
ofninfo.nMaxFile=MAXPATH;
ofninfo.lpstrFileTitle=NULL;
ofninfo.nMaxFileTitle=0;
ofninfo.lpstrInitialDir=".\\";
ofninfo.lpstrTitle=title;
ofninfo.Flags=OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
ofninfo.nFileOffset=0;
ofninfo.nFileExtension=0;
ofninfo.lpstrDefExt="*";
ofninfo.lCustData=0L;
ofninfo.lpfnHook=NULL;
ofninfo.lpTemplateName=NULL;
if(!GetOpenFileName(&ofninfo))
  {
  err=CommDlgExtendedError();
  if(err!=0) 			// 0 value means user selected Cancel
    {
    error(AppName,MB_RETRYCANCEL|MB_ICONERROR|MB_SYSTEMMODAL,
      "File not found...");
    }
  return(0);
  }
strcpy(name,ofninfo.lpstrFile);
return(1);
}


int error(char *hdr,int key,char *fmt, ... )	// error message
{
va_list argptr;
static char str[MAXPATH+80];
int res;

va_start(argptr,fmt);
vsprintf(str,fmt,argptr);
res=MessageBox(0,str,hdr,key);
va_end(argptr);
return(res);
}



